##########################################################################
# configurator - simple config program for midori linux
# written by Claudio Fleiner
#
# Copyright (C) 2000 Transmeta Corporation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require Cwd;    

# Read the user config files and config files to be imported,
# and write the dot files (.config.sh and .config.mk)
package CONFIG;

# This function reads the config file line by line and sets the self hash.
# Must be done before reading the CONFIG files.
sub read_file {
   my $self={ };
   bless $self;
   my $filename=shift;

   defined($filename) || return;
   open(C,$filename) || return; # thats ok: we create it later
   while(<C>) {
      my ($name, $val) = parse_line($_);
      next if (not defined $name);
      $self->{$name} = $val;
   }
   close(C);
   return $self;
}

# write the .config.sh and .config.mk files
sub dot_files {
   # first collect all parameters and create a few new ones
   my %all;
#
# The main directory structure
   $all{'TOPDIR'}=Cwd::getcwd();
   $all{'BASE'}=$ENV{'BASE'};
   $all{'CACHE'}=$ENV{'CACHE'};
   $all{'ETC'}=$ENV{'ETC'};
   $all{'ROOT'}=$ENV{'ROOT'};
   $all{'TOOLS'}=$ENV{'TOOLS'};
   $all{'TARGET'}=$ENV{'TARGET'};
   $all{'BUILD'}=$ENV{'BUILD'};
   $all{'PARTS'}=$ENV{'PARTS'};
   $all{'IMAGE'}=$ENV{'IMAGE'};
   $all{'INITS'}=$ENV{'INITS'};
   $all{'SCRIPTS'}=$ENV{'SCRIPTS'};
   $all{'APPS'}=$ENV{'APPS'};

# make sure to fix the path so that everything is found
   my $p=":".$ENV{'PATH'}.":";
   my $o=":".$ENV{'PATH'}.":";
   foreach my $i ($all{'TOOLS'}."/bin",$all{'TOOLS'}."/usr/X11R6/bin") {
      $o=~ s/:$i://g if($o =~ /:$i:/);
      $p=":$i$p" if($p !~ /:$i:/);
   }
   foreach my $i ("/bin","/sbin","/usr/bin","/usr/sbin" ) {
      $o.="$i:" if($o !~ /:$i:/);
      $p.="$i:" if($p !~ /:$i:/);
   }
   $o=~s/^:(.*):$/$1/;
   $p=~s/^:(.*):$/$1/;

   $all{'ORIGPATH'}=$o;
   $all{'PATH'}=$p;

   MENU::get("")->extract_params(\%all);

# now evaluate everything. We look for $varname $(varname)
# ${varname}, if the variable does not exist we leave it alone.
# There is currently no way to block variable expansion, not
# sure if that is actually needed. 
   my $expanded=1;
   my $count=0;
   while($expanded) {
      $count++;
      if($count >30) {
         die "possible endless variable substitution on parameter $i --> $all{$i}\n";
      }
      $expanded=0;
      my $last;
      foreach my $i (keys %all) {
         if($all{$i} =~ /\$([A-Z][A-Z0-9_]*)/ && defined $all{$1}) {
            $all{$i} =~ s/\$([A-Z][A-Z0-9_]*)/$all{$1}/;
            $last=$i;
            $expanded=1;
         } elsif($all{$i} =~ /\${([A-Z][A-Z0-9_]*)}/ && defined $all{$1}) {
            $all{$i} =~ s/\${([A-Z][A-Z0-9_]*)}/$all{$1}/;
            $expanded=1;
            $last=$i;
         } elsif($all{$i} =~ /\$\(([A-Z][A-Z0-9_]*)\)/ && defined $all{$1}) {
            $all{$i} =~ s/\$\(([A-Z][A-Z0-9_]*)\)/$all{$1}/;
            $expanded=1;
            $last=$i;
         }
      }
   }

# backtick expansion 
   $expanded=1;
   $count=0;
   while($expanded) {
      $count++;
      if($count >30) {
         die "possible endless backtick substitution on parameter $i --> $all{$i}\n";
      }
      $expanded=0;
      my $last;
      foreach my $i (keys %all) {
         if($all{$i} =~ /`([^`]*)`/) {
            $r=`$1`;
            chop $r;
            $all{$i}=~s/`([^`]*)`/$r/;
            $last=$i;
            $expanded=1;
         }
      }
   }

# Note: no more variable substitution after backticks
# Now add the parallel make parameters
   my $make=$ENV{'MAKE'};
   $make="make" if !defined($make) || $make eq "";
   $all{'MAKE'}=$make;
   $p=$all{'PARALLEL_MAKE'};
   if(defined($p) && $p eq "yes") {
      my $count=1;
      if(open(P,"/proc/cpuinfo")) {
         while(<P>) {
            $count ++ if(/^processor/);
         }
         close P;
      }
      $all{'JCOUNT'}=$count;
      $all{'JMAKE'}="$make -w -j $count";
   } else {
      $all{'JCOUNT'}=1;
      $all{'JMAKE'}="$make -w";
   }

# now dump
   open(SH,">.config.sh.new") || die "cannot open .config.sh.new for writing: $!\n";
   open(MK,">.config.mk.new") || die "cannot open .config.mk.new for writing: $!\n";
   foreach my $i (sort keys %all) {
      my $r=$all{$i};
      (my $sh=$r) =~ s/'/'\\''/g;
      (my $mk=$r) =~ s/\n/\\\n/g;
      print SH "$i='$sh'\n" if($i ne "PACKAGE_LIST");
      print MK "$i=$mk\n";
   }
   close MK;
   print SH "export";
   foreach my $i (sort keys %all) {
      print SH " $i";
   }
   print SH "\n";
   close SH;

   system("if cmp -s .config.sh .config.sh.new ; then rm .config.sh.new ; else mv -f .config.sh.new .config.sh ; fi");
   system("if cmp -s .config.mk .config.mk.new ; then rm .config.mk.new ; else mv -f .config.mk.new .config.mk ; fi");
}


# Read a config file and change all parameters that are in there
# as if they would have been edited by hand.
sub import_file {
   my $file=shift;
   my $config=CONFIG::read_file($file);
   MENU->top->import_file($config);
}

# find all configuration files in a directory
# and return a hash   filename --> comment
# with comment being the first comment until an empty line.
sub find_config_files {
   my $dir=shift;
   my $res={ };
   foreach my $i (glob("$dir/*.config")) {
      my $r=open(F,$i);
      if($r) {
         my $c="";
         while(<F>) {
            chop;
            last if(/^\s*$/ || /^[^#]/);
            if(/^#\s*(.*)/) {
               $c.="$1\n";
            }
         }
         close F;
         $$res{$i}=$c;
      }
   }
   return $res;
}

# Parse a line of the form NAME=VALUE
# Knows about " and '
sub parse_line {
   my ($line) = @_;
   my ($name, $raw_val, $val);

   (defined $line) or confess("no $line");

   chomp $line;
   ($line =~ /^\s*([A-Z0-9_]+)\s*=\s*(.*)\s*$/) or return (undef, undef);

   $name = $1;
   $raw_val = $2;
   
   # FIXME: we could use better sanity checking here
   if ($raw_val =~ /^"([^"]*)"$/) { return ($name, $1); };
   if ($raw_val =~ /^'([^']*)'$/) { return ($name, $1); };
   if ($raw_val =~ /^(\S*)/) { return ($name, $1); };

   return (undef, undef);
}

1;

