##########################################################################
# configurator - simple config program for midori linux
# written by Claudio Fleiner
#
# Copyright (C) 2000 Transmeta Corporation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package MENU;


use Carp;

my %menus;

# Delete all menus, useful when reloading CONFIG files
sub clear {
   undef %menus;
}

# return the top menu (with name "");
sub top {
   return MENU::get("");
}

# Return a menu if it exists, or create it
# and connect it to its parent.
# $1: menu name (Config/Runtime/HTTP)
# $2: title
sub get { 
   my $m=shift;
   my $t=shift;
   my $self;

   (defined $m) or $m = '';
   (defined $t) or $t = '';

   if($menus{$m}) {
      $self=$menus{$m};
   } else {
      $self={ };
      bless $self;
      $menus{$m}=$self;
      my $cref=[];
      $self->{CHILDREN}=$cref;
      if($m =~ /^(.*)\/([^\/]*)$/) {
         $self->menu($m);
         $self->name($2);
         $self->parent(MENU::get($1));
      } elsif($m ne "") {
         $self->menu($m);
         $self->name($m);
         $self->parent(MENU::get(""));
      }
   }
   $self->title($t) if($t ne "");
   return $self;
}


# return menu object for a menu
sub find {
   my $m=shift;
   return $menus{$m};
}


# parent: returns the parent of a menu, or nothing
# if used on the top menu entry called ""
sub parent {
   my $self = shift;
   if (@_) { 
      my $p=shift;
      $self->{PARENT} = $p;
      $p->children($self);
   }
   return $self->{PARENT};
}

# return all children or add one
sub children {
   my $self = shift;
   if (@_) { 
      my $cref=$self->{CHILDREN};
      my $child=shift;
      push @$cref,$child;
#      @$cref=sort { $a->name cmp $b->name } ( @$cref ,  @_);
#      $self->{CHILDREN}=$cref;
   }
   return $self->{CHILDREN};
}

# The full menu name (foo/bar/something).
# $1: self
# $2: menu (optional)
# Will set the menu if passed as an argument.
sub menu {
   my $self = shift;
   if (@_) { $self->{MENU} = shift }
   return $self->{MENU};
}

# The last entry of the menu, for foo/bar/something
# it would return 'something'
# $1: self
# $2: name (optional)
# Will set the name if passed as an argument.
sub name {
   my $self = shift;
   if (@_) { $self->{NAME} = shift }
   return $self->{NAME};
}


# read/set title of menu (used in the html menu )
sub title {
   my $self = shift;
   if (@_) { $self->{TITLE} = shift; }
   return $self->name if((not defined $self->{TITLE}) or ($self->{TITLE} eq ""));
   return $self->{TITLE};
}

# Cycle through all children in order to save the data into
# a configuration file.
sub save() {
   my $self=shift;
   my $file=shift;
   my $c=$self->children;
   my $i;
   foreach $i (@$c) {
      $i->save($file);
   }
}


# Import a config file and make the necessary changes to
# all files attached to this menu
sub import_file() {
   my $self=shift;
   my $config=shift;
   my $c=$self->children;
   foreach my $i (@$c) { $i->import_file($config); }
}

# enable all packages that are attached to this menu
sub enable() {
   my $self=shift;
   my $c=$self->children;
   foreach my $i (@$c) { $i->enable(); }
}

# disable all packages that are attached to this menu
sub disable() {
   my $self=shift;
   my $c=$self->children;
   foreach my $i (@$c) { $i->disable(); }
}

# set all paramteres of all packages attached to this
# menu to their default value 
sub to_default() {
   my $self=shift;
   my $c=$self->children;
   foreach my $i (@$c) { $i->to_default(); }
}

# Extract all parameters of all packages attached to this 
# menu into a hash
sub extract_params() {
   my $self=shift;
   my $all=shift;
   my $c=$self->children;
   foreach my $i (@$c) { $i->extract_params($all); }
}

1;
