##########################################################################
# configurator - simple config program for midori linux
# written by Claudio Fleiner
#
# Copyright (C) 2000 Transmeta Corporation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# An ML package contains all the information that is relevant for 
# a specific version of a package. Different versions of the same
# package are grouped together in a PRG object.
package ML;

use Carp;

my %packages;

# clear all packages (useful if everything gets reloaded)
sub clear {
   undef %packages;
}

# Create new package
sub new { 
   my $self = {};
   bless $self;
   $self->{PARAM}={};
   $self->{PARAMORDER}=[];
   $self->{TITLE}='[this ML object is untitled]';
   $self->getpackage(shift);
   return $self;
}

# Return the object describing a particular package.version
sub get {
   my $n=shift;
   my $self=$packages{$n};
   return $self;
}

# full file name (readonly)
sub file {
   my $self = shift;
   die "you cannot set the filename this way, use dir and package instead\n" if(@_);
   return $self->{DIR}.$self->{PACKAGE};
}

# get a paramter object (if there is none a new one is created)
sub param {
   my $self = shift;
   my $p = shift;
   my $ref=$self->{PARAM};
   if($$ref{$p}) {
      return $$ref{$p};
   } else {
      my $param=PARAMETER::new($p,$self);
      $$ref{$p}=$param;
      my $refo=$self->params;
      my $k;
      push @$refo,$param;
      $self->params($refo);
      return $param;
   }
}

# true if any paramter has been edited
sub edited {
   my $self=shift;
   my $p=$self->params;
   foreach my $i (@$p) {
      return 1 if($i->edited);
   }
   return 0;
}

# true if any paramter has a different config file value
# than the default value.
sub changed {
   my $self=shift;
   my $p=$self->params;
   foreach my $i (@$p) {
      return 1 if($i->changed);
   }
   return 0;
}

# true if any paramter is set to an illegal value
sub has_error {
   my $self=shift;
   my $p=$self->params;
   foreach my $i (@$p) {
      return 1 if(!$i->check_value);
   }
   return 0;
}

# optionally set the title. In any case return the currently 
# used title.
sub title {
   my $self = shift;
   if (@_) { $self->{TITLE} = shift }
   return $self->val if(!defined($self->{TITLE}) || $self->{TITLE} eq "");
   return $self->{TITLE};
}

# Version of the CONFIG file type used. Currently unused.
sub version {
   my $self = shift;
   if (@_) { $self->{VERSION} = shift }
   return $self->{VERSION};
}

# Description of this package (from the CONFIG file)
sub description {
   my $self = shift;
   if (@_) { $self->{DESCRIPTION} = shift }
   return $self->{DESCRIPTION};
}

# Name of the menu to which this package belongs too.
# WARNING: if two different versions of the same package
# belong to two different menus chaos will happen.
sub menuname {
   my $self = shift;
   if (@_) { $self->{MENUNAME} = shift }
   return $self->{MENUNAME};
}

# Packager of this particular package
sub author {
   my $self = shift;
   if (@_) { $self->{AUTHOR} = shift }
   return $self->{AUTHOR};
}

# email address of packager
sub email {
   my $self = shift;
   if (@_) { $self->{EMAIL} = shift }
   return $self->{EMAIL};
}

# URL where this package can be downloaded
sub url {
   my $self = shift;
   if (@_) { $self->{URL} = shift }
   return $self->{URL};
}

# return list of paramters
sub params {
   my $self = shift;
   if (@_) { $self->{PARAMORDER} = shift }
   return $self->{PARAMORDER};
}


# directory where the package was found (/opt/midori/apps)
sub dir {
   my $self = shift;
   if (@_) { $self->{DIR} = shift }
   return $self->{DIR};
}

# package name (linux-2.4.0-test2_ML2.4-1.mlb)
sub package {
   my $self = shift;
   if (@_) { $self->{PACKAGE} = shift }
   return $self->{PACKAGE};
}

# value to which variable must be set (linux-2.4.0-test12)
sub val {
   my $self = shift;
   if (@_) { 
      $self->{VAL} = shift;
      $packages{$self->{VAL}}=$self;
   }
   return $self->{VAL};
}

# package name (linux)
sub name {
   my $self = shift;
   if (@_) { $self->{NAME} = shift }
   return $self->{NAME};
}

# variable name (LINUX_SRC)
sub var {
   my $self = shift;
   if (@_) { $self->{VAR} = shift }
   return $self->{VAR};
}

# variable name for file (LINUX_SRC_PACKAGE)
sub filevar {
   my $self = shift;
   return $self->var."_PACKAGE";
}

# children (always empty)
sub children {
   return [ ];
}

# Full menu name (configuration/linux-2.4.0-test12)
sub menu {
   my $self = shift;
   return $self->parent->menu."/".$self->val;
}

# menu where this package will show up
sub parent {
   my $self = shift;
   if (@_) { $self->{PARENT} = shift }
   return $self->{PARENT};
}

# the config file (array of string)
sub config {
   my $self = shift;
   if (@_) { $self->{CONFIG} = shift }
   return $self->{CONFIG};
}

# grep for old style menu definitions (they all start with a '#@')
sub config_mgrep {
   my $self = shift;
   my $pattern = shift;
   my @res;
   my $i;
   my $r;
   my $ref=$self->config;

   foreach $i (@$ref) {
      push @res,$1 if($i =~ /^#@\s*$pattern\s+(.*)/);
   }
   return @res;
}

# Read the config file of a package
sub getpackage($) {
   my $self=shift;
   my $file=shift;

   (defined $self) or confess("\$self undefined!\n");
   (defined $file) or confess("\$file undefined!\n");


# if you change this expression you must also change the one in functions.sh
   if($file =~ /^(.*\/)?([A-Za-z][A-Za-z0-9+.-]*?)(-([0-9][^_]*))?(_(.*))?(\.mlz)$/) {
      my $dir="";
      my $name="";
      my $ver="";
      my $fver="";
      my $patch="";
      my $fpatch="";
      my $ending="";

      $dir=$1 if(defined($1));
      $name=$2 if(defined($2));
      $fver=$3 if(defined($3));
      $ver=$4 if(defined($4));
      $fpatch=$5 if(defined($5));
      $patch=$6 if(defined($6));
      $ending=$7 if(defined($7));

      $self->dir($dir);
      $self->package("$name$fver$fpatch$ending");
      $self->val("$name$fver");
      $self->name("$name");
      (my $var = $name."_SRC") =~ tr /[a-z]+.-/[A-Z]___/;
      $self->var($var);
   } else {
      die "package name $file cannot be interpreted\n";
   }
   my $configfile=$self->val."/CONFIG";
   my $cfcache="$MAIN::conf/" . $self->val;
   my $reload=1;
   if(-f $cfcache)  {
      my $mtimep = (stat($self->file))[9]; # mtime
      my $mtimec = (stat($cfcache))[9]; # mtime
      $reload=0 if($mtimec>=$mtimep);
   }
   if($reload) {
      print "tar xOzf ".$self->file." $configfile\n";
      system("tar xOzf ".$self->file." $configfile 2> /dev/null > $cfcache");
   }

   my $ref=[];
   $self->config($ref);
   open(C,$cfcache) || die "cannot open $cfcache for reading: $!\n";
   while(<C>) {
      chop;
      push @$ref,$_;
   }
   close C;

   (defined $cfcache) or confess("\$cfcache not defined!\n");

   $self->get_param;
   return $self->params;
}

# get the menu position for the package if it has not
# been defined already (only used for backwards compatibiliy)
sub get_menu() {
   my $self = shift;
   my $i;

   my $mn=$self->menuname;
   if(!defined($mn)) {
      $mn="misc";
      my $ref=$self->config;
      foreach $i ($self->config_mgrep("MENU")) {
         $mn=$i;
         last;
      }
   }
   $self->parent(PRG::get($mn."/".$self->name));
   my $m=$self->parent;
   $m->children($self);
}

# get menu titles that may have been defined in the CONFIG file (only used for
# backwards compatibiliy)
sub get_menutitle() {
   my $self = shift;
   my $ref=$self->config;
   foreach $i ($self->config_mgrep("MENUTITLE")) {
      if($i =~ /^(\S+)\s+(.*)/) {
         my $m=MENU::get($1);
         $m->title($2);
      }
   }
}


# create parameter
# $1=self
# $2=name
# $3=description
# $4=type
# $5=default
sub create_param {
   my ($self,$name,$descr,$type,$default)=@_;

   $default="" if(!defined($default));
   $type="" if(!defined($type));
   $descr="" if(!defined($descr));
   my $p=$self->param($name);
   $p->default($default);
   if(exists($MAIN::user_config->{$name})) {
      $p->config($MAIN::user_config->{$name});
   } else {
      $p->config($p->default);
   }
   $p->edit($p->config);
   $p->comment($descr);
   $p->type($type);
#   print "PARAM: name=".$p->name;
#   print " edit=".$p->edit;
#   print " config=".$p->config;
#   print " default=".$p->default;
#   print " type=".$p->type;
#   print "\n";
   return $p;
}

# remove string delimiters
sub remove_string_delim {
   my $s=shift;
   $s=~s/^\s*(.*\S)\s*$/$1/;
   return $1 if($s =~ /'(.*)'/);
   return $1 if($s =~ /"(.*)"/);
   return $s;
}


# get all paramters for new config file versions
sub get_param {
   my $self = shift;
   my $c=$self->config;

   return $self->get_old_param if(@$c==0 || $$c[0] !~ /^^(# )?Title:/);

   my $value="";
   my $name="";
   my $descr="";
   my $type="STRING";
   my $default="";
   my $attr="";
   my $line=0;

   # add empty line to force processing of last attribute
   foreach my $i (@$c,"") {
      $line++;
      #        vv definition of new attr                 vv empty line      vv  setting parameter
      if($i =~ /^(# )?([A-Za-z0-9_]*):\s*(.*)$/ || $i =~ /^(# )?\s*$/ || $i =~ /^\*s[A-Z0-9_]+\s*=/) {
         if($attr ne "") {
            if($attr eq "Title") {
               $self->title($value);
            } elsif($attr eq "Url" || $attr eq "URL") {
               $self->url($value);
            } elsif($attr eq "Author") {
               $self->author($value);
            } elsif($attr eq "Packager") {
               $self->author($value);
            } elsif($attr eq "Maintainer") {
               $self->author($value);
            } elsif($attr eq "Email") {
               $self->email($value);
            } elsif($attr eq "Menu") {
               if($value =~ /^\s*(\S+)\s*$/) {
                  $self->menuname($1);
               } else {
                  print $self->val."/CONFIG:$line: strange menu line '$value'\n";
               }
            } elsif($attr eq "Menutitle") {
               if($value =~ /^(\S+)\s+(.*)/) {
                  my $m=MENU::get($1);
                  $m->title($2);
               } else {
                  print $self->val."/CONFIG:$line: strange menutitle line '$value'\n";
               }
            } elsif($attr eq "Version") {
               $self->version($value);
            } elsif($attr eq "Description") {
               if(!defined($name) || $name eq "") {
                  $self->description($value);
               } else {
                  $descr=$value;
               }
            } elsif($attr eq "Type") {
               $type=$value;
            } elsif($attr eq "Default") {
               $default=ML::remove_string_delim($value);
            } elsif($attr eq "Name") {
               $self->create_param($name,$descr,$type,$default) if($name ne "");
               $name=$value;
               $type="STRING";
               $descr="";
               $default="";
            } else {
               print $self->val."/CONFIG:$line: Unkown field $attr\n";
            }
               
         }
         $attr="";
         $value="";
      }
      if($i =~ /^(# )?([A-Za-z0-9_]*):\s*(.*)$/) {
         $attr=$2;
         $value=$3;
      } elsif($i =~ /^(# )?\s+(.*)$/) {
         $value.="\n$2";
      } elsif($i =~ /^\s*([A-Z0-9_]+)\s*=\s*(.*)$/) {
         if($name ne $1) {
            print $self->val."/CONFIG:$line: name mismatch\n";
            next;
         }
         my $v=$2;

      }
   }
   $self->create_param($name,$descr,$type,$default) if($name ne "");
   $self->get_menu;
   $self->get_menutitle;
}

# get all paramters for old config file versions
sub get_old_param {
   my $self = shift;
   $self->get_menu;
   $self->get_menutitle;

   my $c=$self->config;
   my $lastcomment="";
   my $first=1;
   foreach my $i (@$c) {
      next if($i =~ /^#@/); # skip menu code
      if($i =~ /^\s*$/) {
         $self->title($lastcomment) if($first==1);
         $first=0;
         $lastcomment="";
         next;
      }
      if($i =~ /^#\s*(.*)/) {
         $lastcomment.=$1."\n";
         next;
      }

      my ($name, $value) = CONFIG::parse_line($i);
      if (defined($name)) {
         $self->create_param($name,$lastcomment,$type,$value) if($name ne "");
         $lastcomment="";
      }
   }
   $self->title($lastcomment) if($first==1);
}


# Write all parameters to a config file
sub save {
   my $self=shift;
   my $file=shift;
   my $i;
   my $comment=$self->parent->edit ne $self->val && $self->parent->edit ne "yes"; # non active packages get everything commented out

   print $file ("#" x 79)."\n";
   print $file "# ".$self->val.(" " x (76-length($self->val)))."#\n";

   my $c=$self->config;
   foreach $i (@$c) {
      next if($i =~ /^#@/);
      if($i =~ /^\s*([A-Z0-9_]+)=/) {
         my $p=$self->param($1);
         if($comment) {
            print $file "# ".$p->name."=\"".$p->default."\"\n";
         } else {
            print $file "# " if($p->edit eq $p->default);
            print $file $p->name."=\"".$p->edit."\"\n";
         }
      } else {
         print $file "$i\n";
      }
   }
   print $file "#\n";
   print $file "# ".$self->val.(" " x (76-length($self->val)))."#\n";
   print $file ("#" x 79)."\n";
}

# Import a config file
sub import_file {
   my $self=shift;
   my $config=shift;
   my $p=$self->params;
   foreach my $i (@$p) {
      $i->edit($config->{$i->name}) if(defined($config->{$i->name}));
   }
}

# switch to default values for all paramters.
sub to_default() {
   my $self=shift;
   my $param=$self->params;
   foreach $i (@$param) {
      $i->edit($i->default);
   }
}

1;
