##########################################################################
# configurator - simple config program for midori linux
# written by Claudio Fleiner
#
# Copyright (C) 2000 Transmeta Corporation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# a PRG object combines all the different versions of the same package.
# In most cases it will only know about one, but in other instances you
# may have more than one version available.
package PRG;

@ISA=qw( MENU );

# Return the PRG object for a package
sub get { 
   my $self= MENU::get(@_);
   bless $self;
   return $self;
}


# variable name (LINUX_SRC)
sub var() {
   my $self=shift;
   my $c=$self->children;
   return $$c[0]->var;
}
#
# variable name for file (LINUX_SRC_PACKAGE)
sub filevar {
   my $self = shift;
   return $self->var."_PACKAGE";
}

# return the current active child name
sub edit {
   my $self = shift;
   if (@_) { $self->{EDIT} = shift }
   if ((not defined $self->{EDIT}) or ($self->{EDIT} eq '')) { $self->{EDIT} = $self->config }
   return $self->{EDIT};
}

# return child with given name
sub child {
   my $self = shift;
   my $name = shift;
   my $c=$self->children;
   my $i;
   foreach $i (@$c) {
      return $i if($i->val eq $name);
   }

   return 0;
}


# return the active child according to the config file.
# if the config file says nothing the latest child (according to
# the file name) is returned
sub config {
   my $self = shift;
   my $c = $MAIN::user_config->{$self->var};
   if((not defined($c)) || ($c eq "")) {
      my $n = $MAIN::user_config->{'DEFAULT_INSTALL'};
      if((not defined($n))  || $n eq "yes") {
         return $self->latest;
      } else {
         return "no";
      }
   }

   return $self->latest if((not defined $c) or ($c eq "") or ($c eq "yes"));
   return $c;
}

# compare two file versions and try to guess which
# version is the newer one
sub cmpversion {
   my ($a, $b) = @_;
   my ($i, @A, @B);

   @A = split('[\.-]', $a);
   @B = split('[\.-]', $b);
   for ($i = 0; $i <= $#A; $i++) {
      if($A[$i] ne $B[$i]) {
         return ($A[$i] <=> $B[$i]) if ($A[$i] =~ /^\d/ && $B[$i] =~ /^\d/ && $A[$i] != $B[$i]);
         return ($A[$i] cmp $B[$i]);
      }
   }
   return ($a cmp $b);
}


# return the latest (default) child
sub latest {
   my $self = shift;
   my $r=$self->children;
   my $c=$$r[0];
   foreach my $i (@$r) {
      $c=$i if(cmpversion($c->val,$i->val)<0);
   }
   return $c->val;
}


# Save configuration to a config file
sub save {
   my $self=shift;
   my $file=shift;
   my $children=$self->children;
   my $i;

   print $file "\n\n".("#" x 79)."\n";
   print $file ("#" x 79)."\n";
   (my $n=$self->name) =~ tr/[a-z]/[A-Z]/;
   print $file "### ".$n.(" " x (72-length($n)))."###\n";
   print $file ("#" x 79)."\n";
   if(@$children>1 || $self->edit eq "no") {
      print $file "# Currently available packages:\n";
      foreach $i (@$children) {
         print $file "# - ".$i->val."\n";
      }
      print $file $self->var."=".$self->edit."\n\n";
   } else {
#print $file "# only one version of this package found:\n";
      print $file "# only one package is available:\n";
      print $file "# - ".$$children[0]->val."\n";
      print $file $self->var."=yes\n\n";
   }

   foreach $i (@$children) {
      $i->save($file);
   }
   
   print $file ("#" x 79)."\n";
   print $file ("#" x 79)."\n";
}

# import a config file
# $1: self
# $2: hash of config variables set in config file
sub import_file() {
   my $self=shift;
   my $config=shift;
   my $c=$self->children;
   my $src=$config->{$self->var};
   if(defined($src)) {
      if($src eq "no") {
         $self->edit("no");
      } elsif($src eq "yes") {
         if($self->edit eq "no") {
            $self->edit($self->latest);
         }
      } else {
         $self->edit($src);
      }
   }
   foreach my $i (@$c) { $i->import_file($config); }
}


# enable this package
sub enable() {
   my $self=shift;
   $self->edit($self->latest);
}

# disable this package
sub disable() {
   my $self=shift;
   $self->edit("no");
}

# extract all parameters into a hash.
# If the user did not yet select one package version in the case
# where more than one is available a warning message is written to
# the file .warning. The same happens if a version was specified
# that does not exist.
sub extract_params() {
   my $self=shift;
   my $all=shift;
   if($self->edit eq "no") {
      $$all{$self->var}="no";
   } else {
      my $ch=$self->children;
      my $e = $MAIN::user_config->{$self->var};
      if((!defined($e) || $e eq "yes") && (@$ch)>1) {
         if(open(W,">>.warning")) {
            my $i=" " x (10+length($self->name));
            print W "Package ".$self->name.": several versions are available\n".$i."so you need to specify which one you want:\n";
            foreach $cc (@$ch) {
               print W "$i- ".$cc->val."\n";
            }
            close W;
         }
         return;
      }
      my $c=$self->child($self->edit);
      if($c eq 0) {
         if(open(W,">>.warning")) {
            my $i=" " x (10+length($self->name));
            print W "Package ".$self->name.": requested version ".$self->edit."\n".$i."not found. Available packages:\n";
            foreach $cc (@$ch) {
               print W "$i- ".$cc->val."\n";
            }
            close W;
         }
         return;
      }
      my $p=$c->params;
      $$all{$self->var}=$c->val;
      $$all{$self->filevar}=$c->file;
      $$all{'PACKAGE_LIST'}.=" ".$c->file;
      foreach my $i (@$p) {
         $$all{$i->name}=$i->edit;
      }
   }
}

# Change all parameter values to their default value as
# defined in the CONFIG file.
sub to_default() {
   my $self=shift;
   $self->edit($self->latest);
   my $c=$self->children;
   foreach my $i (@$c) { $i->to_default(); }
}

1;
