##########################################################################
# configurator - simple config program for midor linux
# written by Claudio Fleiner
#
# Copyright (C) 2000 Transmeta Corporation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package ML;


# Print the input mask for all paramteres
# $1: self
# $2: output stream
sub print_param() {
   my $self=shift;
   my $file=shift;
   my $parms=$self->params;
   if(@$parms>=1) {
      my $j;
      print $file "<tr><td align=left><small>Parameter Name</td><td align=left><small>Current Value</small></td><td align=left><small>Default Value</small></td><td align=left><small>Value in Config File</small></td></tr>\n";
      foreach $j (@$parms) {
         if(!$j->hidden) {
            print $file "<tr><td align=left ".($j->valign?"valign=top":"")." bgcolor=".$j->get_color;
            my $fn="/packhelp.cgi?ml=".SERVER::eurl($self->val)."#".SERVER::eurl($j->name);
            print $file "><a href=\"$fn\" target=help>".$j->name."</a><td>";
            $j->printhtml($file);
            print $file "</td><td ".($j->valign?"valign=top":"").">";
            print $file "<small>".$j->default."</small>\n";
            print $file "</td><td ".($j->valign?"valign=top":"").">";
            print $file "<small>".$j->config."</small>\n";
            print $file "</td></tr>";
         }
      }
   }
}

# Print the help screen (the information stored in the CONFIG file)
# $1: self
# $2: output stream
sub printhelp {
   my $self=shift;
   my $file=shift;

   print $file "<html><head><title>".$self->title."</title></head><body bgcolor=ffffff>\n";
   print $file "<h1>".$self->title."</h1>\n";
   if(defined($self->author) || defined($self->url) || defined($self->email)) {
      print $file "<table border=0>\n";
      print $file "<tr><td align=right>Packager:</td><td><b>".$self->author."</b></td></tr>\n" if(defined($self->author));
      print $file "<tr><td align=right>Email:</td><td><b><a href=mailto:".$self->email.">".$self->email."</a></b></td></tr>\n" if(defined($self->email));
      print $file "<tr><td align=right>URL:</td><td><b><a target=_blank href=\"".SERVER::eurl($self->url)."\">".$self->url."</a></b></td></tr>\n" if(defined($self->url));
      print $file "</table><p>\n";
   }
   print $file $self->description."\n" if(defined($self->description));

   my $p=$self->params;
   foreach $i (@$p) {
      $i->printhelp($file) if(!$i->hidden);
   }
}

# Print all the changes that were made 
# $1: self
# $2: output stream
# $3: title to print if something changed
sub print_changes {
   my $self=shift;
   my $file=shift;
   my $title=shift;
   my $param=$self->params;
   my $edit=0;
   my $i;
   foreach $i (@$param) {
      $edit=1 if($i->edited || $i->changed || !$i->check_value);
   }
   if($edit) {
      print $file $title;
      print $file "<table border=1><tr><th>Parameter</th><th>New Value</th><th>Old Value</th><th>Default Value</th></tr>\n";
      foreach $i (@$param) {
         if($i->edited || $i->changed) {
            print $file "<tr><td bgcolor=".$i->get_color;
            print $file ">".$i->name."</td><td>".$i->edit."&nbsp;</td><td>".$i->config."</td><td>".$i->default."&nbsp;</td></tr>\n";
         }
      }
      print $file "</table>\n";
   }
}

# Print the changed that would happen if some config file would be imported
# $1: self
# $2: output stream
# $3: Title to print (only printed if something else changes too)
# $4: configuration to test against
sub print_whatif {
   my $self=shift;
   my $file=shift;
   my $title=shift;
   my $conf=shift;
   my $param=$self->params;
   my $edit=0;
   my $i;
   foreach $i (@$param) {
      $edit=1 if(defined($conf->{$i->name}) && $conf->{$i->name} ne $i->edited);
   }
   if($edit) {
      print $file $title;
      print $file "<table border=1><tr><th>Parameter</th><th>Imported Value</ht><th>Current Value</th><th>Old Value</th><th>Default Value</th></tr>\n";
      foreach $i (@$param) {
         if(defined($conf->{$i->name}) && $conf->{$i->name} ne $i->edit) {
            my $val=$conf->{$i->name};
            print $file "<tr><td ";
            if(!$i->check_value(val)) {
               print $file "bgcolor=ff0000";
            } elsif($val eq $i->default) {
               print $file "bgcolor=77ff77";
            } elsif($val eq $i->config) {
               print $file "bgcolor=ffff00";
            } else {
               print $file "bgcolor=ffaa00";
            }
            print $file ">".$i->name."</td><td>".$conf->{$i->name}."</td><td>".$i->edit."&nbsp;</td><td>".$i->config."</td><td>".$i->default."&nbsp;</td></tr>\n";
         }
      }
      print $file "</table>\n";
   }
}

# Print the parameters that are different from their default value
sub print_whatif_default {
   my $self=shift;
   my $file=shift;
   my $title=shift;
   my $param=$self->params;
   print "print_whatif_default: ".$self->name."\n";
   my $edit=0;
   foreach $i (@$param) {
      $edit=1 if($i->edit ne $i->default || $i->config ne $i->default);
   }
   if($edit) {
      print $file $title;
      print $file "<table border=1><tr><th>Parameter</th><th>Default Value</ht><th>Current Value</th><th>Old Value</th></tr>\n";
      foreach $i (@$param) {
         if($i->edit ne $i->default || $i->config ne $i->default) {
            print $file "<tr><td ";
            print $file "bgcolor=77ff77";
            print $file ">".$i->name."</td><td>".$i->default."</td><td>".$i->edit."&nbsp;</td><td>".$i->config."&nbsp;</td></tr>\n";
         }
      }
      print $file "</table>\n";
   }
}

# print menu entry (not used)
sub print_menu() { }

1;
