##########################################################################
# configurator - simple config program for midor linux
# written by Claudio Fleiner
#
# Copyright (C) 2000 Transmeta Corporation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package PARAMETER;

my @paramtypes;

# Print input form element for a paramter. Most subtypes overwrite
# this function, its only here for backwards compatibility.
sub printhtml {
   my $self = shift;
   my $file = shift;
   if(!defined($self->type)) {
      print "undefined type for ".$self->name."\n";
      $self->{TYPE}="";
   } 
   if($self->type eq "YESNO") {
      print $file "<select name=".$self->name."><option value=no";
      print $file " selected" if($self->edit eq "no");
      print $file ">no<option value=yes";
      print $file " selected" if($self->edit eq "yes");
      print $file ">yes</select>";
   } elsif($self->type eq "TRUEFALSE") {
      print $file "<select name=".$self->name."><option value=false";
      print $file " selected" if($self->edit eq "false");
      print $file ">false<option value=true";
      print $file " selected" if($self->edit eq "true");
      print $file ">true</select>";
   } else {
      print $file "<input type=text name=".$self->name." value=\"".$self->edit."\">";
   }
   print $file "\n";
}

# return 1 if the name and default values should be aligned vertically
sub valign { return 0; }

# return 1 if this option should be hidden
sub hidden { return 0; }

# print the acceptable values (in a table row with two fields,
# short description and the acceptable values).
# Example: <tr><td>Acceptable Values:</td><td>1, 2 or 3</td></tr>
# $1: self
# $2: output stream
sub printacceptable { }
 
sub printhelp {
   my $self=shift;
   my $file=shift;
   print $file "<h3><a name=".$self->name."></a>".$self->name."</h3>\n";
   print $file $self->comment if(defined($self->comment));
   print $file "<p><table border=1><tr><td valign=top>Default Value</td><td>".$self->default."&nbsp;</td></tr>\n";
   $self->printacceptable($file);
   print $file "<tr><td valign=top>Config File Value</td><td>".$self->config."&nbsp;</td></tr>\n" if($self->config ne $self->default);
   print $file "<tr><td valign=top>Edited Value</td><td>".$self->edit."&nbsp;</td></tr>\n" if($self->edit ne $self->config);
   print $file "</table></p>\n";
}

# return color to be used for print parameter
sub get_color {
   my $self=shift;
   if($self->check_value) {
      if($self->edited) {
         if($self->edit ne $self->default) {
            return "#ffaa00";
         } else {
            return "#77ff77";
         }
      } elsif($self->changed) {
         return "#ffff00";
      }
   } else {
      return "#ff0000";
   }
   return "#ffffff";
}


# correct fields after they have been returned by the broswer
sub fixfields {  }


##########################################################################
## Readonly parameters (cannot be changed via the webconfig interface)
package PARAM_READONLY;

sub valign { return 1; }

sub printacceptable { 
   my $self=shift;
   my $file=shift;
   print $file "<tr><td valign=top>Acceptable Values</td><td>This value cannot be changed</td></tr>\n";
}

sub printhtml {
   my $self = shift;
   my $file = shift;

   print $file "<input type=hidden name=".$self->name." value=\"".$self->edit."\">".$self->edit;
}


##########################################################################
# Hidden parameters are not visible in the webconfig interface
package PARAM_HIDDEN;

sub hidden { return 1; }
sub valign { return 1; }

sub printacceptable { 
   my $self=shift;
   my $file=shift;
   print $file "<tr><td valign=top>Acceptable Values</td><td>This value cannot be chanegd</td></tr>\n";
}

sub printhtml {
   my $self = shift;
   my $file = shift;

   print $file "<input type=hidden name=".$self->name." value=\"".$self->edit."\">".$self->edit;
}



##########################################################################
# REGEX is used for all types that expect a simple string as input. Even
# STRING, INTEGER and FLOAT use this package
package PARAM_REGEX;

sub printacceptable { 
   my $self=shift;
   my $file=shift;
   print $file "<tr><td valign=top>Acceptable Values</td><td>Regular Expression: '".$self->regex."'</td></tr>\n" if($self->realtype eq "REGEX");
   print $file "<tr><td valign=top>Acceptable Values</td><td>Any String</td></tr>\n" if($self->realtype eq "STRING");
   print $file "<tr><td valign=top>Acceptable Values</td><td>Integer</td></tr>\n" if($self->realtype eq "INTEGER");
   print $file "<tr><td valign=top>Acceptable Values</td><td>Float</td></tr>\n" if($self->realtype eq "FLOAT");
}

sub printhtml {
   my $self = shift;
   my $file = shift;
   my $len=length($self->edit)+5;
   $len=20 if($len<20);

   print $file "<input type=text name=".$self->name." size=$len value=\"".$self->edit."\">";
}

##########################################################################
# ENUM lets the user select ONE entry from a list
package PARAM_ENUM;

sub printacceptable { 
   my $self=shift;
   my $file=shift;
   print $file "<tr><td valign=top>Acceptable Values</td><td>One of:<table border=0>";
   my $o=$self->options;
   my $or=$self->optionsorder;
   foreach my $i (@$or) {
      print $file "<tr><th align=left>$i</td><td>".$$o{$i}."</td></tr>\n";
   }
   print $file "</table></td></tr>\n";
}

sub printhtml {
   my $self = shift;
   my $file = shift;
   my $o=$self->options;
   my $or=$self->optionsorder;

   if($self->radio) {
      foreach my $i (@$or) {
         print $file "<input type=radio name=".$self->name." value=\"$i\" ".($i eq $self->edit?"checked":"").">$i".($$o{$i} eq ""?"":": ".$$o{$i})."<br>\n";
      }
   } else {
      print $file "<select name=".$self->name." size=1>";
      foreach my $i (@$or) {
         print $file "<option value=\"$i\" ".($i eq $self->edit?"selected":"").">$i".($$o{$i} eq ""?"":": ".$$o{$i})."</option>\n";
      }
      print $file "</select>\n";
   }

}


##########################################################################
# LIST: let the user select 0 or more entries from a list
package PARAM_LIST;

sub printacceptable { 
   my $self=shift;
   my $file=shift;
   print $file "<tr><td valign=top>Acceptable Values</td><td>Zero or more:<table border=0>";
   my $o=$self->options;
   my $or=$self->optionsorder;
   foreach my $i (@$or) {
      print $file "<tr><th align=left>$i</td><td>".$$o{$i}."</td></tr>\n";
   }
   print $file "</table></td></tr>\n";
}

sub valign { return 1; }

sub printhtml {
   my $self = shift;
   my $file = shift;
   my $o=$self->options;
   my $or=$self->optionsorder;

   if($self->radio) {
      my $s=$self->edit;
      foreach my $i (@$or) {
         print $file "<input type=checkbox name=".$self->name." value=\"$i\" ".(" $s " =~ / $i / ?"checked":"").">$i".($$o{$i} eq ""?"":": ".$$o{$i})."<br>\n";
      }
   } else {
      print $file "<select name=".$self->name." MULTIPLE>";
      my $s=$self->edit;
      foreach my $i (@$or) {
         print $file "<option value=\"$i\" ".(" $s " =~ / $i / ?"selected":"").">$i".($$o{$i} eq ""?"":": ".$$o{$i})."</option>\n";
      }
      print $file "</select>\n";
   }

}


##########################################################################
# PERL package, no special treatment as everything is done the main package
# already.
package PARAM_PERL;
1;
